#include "cora_fit.h"

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

int read_spec (char *path, int s_flag)
{
  FILE * fd;
  char line[BUFSIZ];
  int count=0, lines=0, items = 0, theline=0;/* ibgs=0, i; */
  char * ptr;
  double * pwave, * psp, * perr, * pbkgd, * pbkerr, *pbg0;

  if (NULL == (fd = fopen(path, "r")))
    {
      fprintf(stderr, 
	      "* Warning *: fopen: %s: File not found\n", path);
      return(-1);
    }

  while (fgets(line, BUFSIZ-1, fd))
    if (line[0] != '#') ++lines;
 
  rewind(fd);

  vec_alloc(&wave,lines);
  vec_alloc(&sp,lines);
  vec_alloc(&err,lines);
  vec_alloc(&bkgd,lines);
  vec_alloc(&bkerr,lines);
  vec_alloc(&bg0,lines);
 
  if (wave.vec == NULL || sp.vec == NULL || err.vec == NULL || 
      bkgd.vec == NULL)
    {
      fprintf(stderr,"* Warning *: Out of memory\n");
      return(-1);
    }

  pwave  = &(wave.vec[0]);
  psp    = &(sp.vec[0]);
  perr   = &(err.vec[0]);
  pbkgd  = &(bkgd.vec[0]);
  pbkerr = &(bkerr.vec[0]);
  pbg0   = &(bg0.vec[0]);


  while (fgets(line, BUFSIZ-1, fd) && count < lines)
    {
      ++theline;
      if (line[0] != '#')
        {
	  ptr = strtok (line, " ");
	  if (ptr)
	    {
	      *pwave = atof(ptr);        ++items;
	      ptr = strtok (NULL, " ");
	      if (ptr)
		{
		  *psp = atof(ptr);          ++items;
		  ptr = strtok (NULL, " ");
		  if (ptr)
		    {
		      *perr = atof(ptr);         ++items;
		      ptr = strtok (NULL, " ");
		      if (ptr)
			{
			  *pbkgd = atof(ptr);        ++items;
			  *pbg0 = *pbkgd;          
			  ptr = strtok (NULL, " ");
			  if (ptr)
			    {
			      *pbkerr = atof(ptr);       ++items;
			    }
			}
		    }
		}
	    }

	  if (items == 2)
	    {
	      *perr=sqrt(*psp);
	      *pbkgd=0.;
	      *pbkerr=0.;
	      *pbg0=0.;
	    }
	  if (items == 3)
	    {
	      *pbkgd=*perr;
	      *pbkerr=sqrt(*pbkgd);
	      *pbg0=*pbkgd;
	      *perr=sqrt(*psp);
	    }
	  if (items == 5 || items == 2 || items == 3)
	    {
		++pwave; ++psp; ++perr; ++pbkgd; ++pbkerr; ++pbg0;
		++count;
		items = 0;
	    }
        }
	 else if (count == 0)
	    {
		ptr = strtok (line, "#");
		exposure=atof(ptr);
	    }
    }
 
fclose(fd);

  if (min(&sp) < 0.)
    {
      fprintf(stderr, "* Error *: A background subtracted spectrum is not allowed: %s.\n",path);
      exit(8);
    }

  if (count != lines)
    {
      fprintf(stderr, "* Warning *: Race condition detected in %s.\n",path);
      return(-1);
    }
  else
    {
      if (verbose)
	fprintf(stdout, "Message      : Total: %d datapoints\n", count);
    }

/*  for (lines=0;lines<count;++lines)  if (bg0.vec[lines]>sp.vec[lines]) ++ibgs;
  if (ibgs>count/2.)
    {
     printf("* Warning *: more than half of the bins\n");
     printf("*         *   contain more counts in the background\n");
     printf("*         *   than in the spectrum\n");
     printf("*   this might indicate that columns in the spectrum file are mixed up.  *\n");
     fd = fopen ("spectrum.swap","w");
     for (i=0;i<wave.len;++i)
        fprintf(fd,"%8.3f %4.2f %4.2f %4.2f %4.2f\n",wave.vec[i],bkgd.vec[i],bkerr.vec[i],sp.vec[i],err.vec[i]);
     fclose(fd);
     printf("* <<<< a file 'spectrum.swap' is provided now with swapped columns >>>>\n");
    }
 */

  return (count);
} 
